@@ -97,22 +97,18 @@ static void OpenCopyConnections(CopyStmt *copyStatement,
 								ShardConnections *shardConnections, bool stopOnFailure,
 								bool useBinaryCopyFormat);
 
-static bool CanUseBinaryCopyFormat(TupleDesc tupleDescription);
 static bool BinaryOutputFunctionDefined(Oid typeId);
 static List * MasterShardPlacementList(uint64 shardId);
 static List * RemoteFinalizedShardPlacementList(uint64 shardId);
-
 static void SendCopyBinaryHeaders(CopyOutState copyOutState, int64 shardId,
 								  List *connectionList);
 static void SendCopyBinaryFooters(CopyOutState copyOutState, int64 shardId,
 								  List *connectionList);
-
 static StringInfo ConstructCopyStatement(CopyStmt *copyStatement, int64 shardId,
 										 bool useBinaryCopyFormat);
 static void SendCopyDataToAll(StringInfo dataBuffer, int64 shardId, List *connectionList);
 static void SendCopyDataToPlacement(StringInfo dataBuffer, int64 shardId,
 									MultiConnection *connection);
-static void EndRemoteCopy(int64 shardId, List *connectionList, bool stopOnFailure);
 static void ReportCopyError(MultiConnection *connection, PGresult *result);
 static uint32 AvailableColumnCount(TupleDesc tupleDescriptor);
 static int64 StartCopyToNewShard(ShardConnections *shardConnections,
@@ -821,10 +817,19 @@ OpenCopyConnections(CopyStmt *copyStatement, ShardConnections *shardConnections,
 		ShardPlacement *placement = (ShardPlacement *) lfirst(placementCell);
 		char *nodeUser = CurrentUserName();
 		MultiConnection *connection = NULL;
-		uint32 connectionFlags = FOR_DML | CONNECTION_PER_PLACEMENT;
+		uint32 connectionFlags = FOR_DML;
 		StringInfo copyCommand = NULL;
 		PGresult *result = NULL;
 
+		/*
+		 * Make sure we use a separate connection per placement for hash-distributed
+		 * tables in order to allow multi-shard modifications in the same transaction.
+		 */
+		if (placement->partitionMethod == DISTRIBUTE_BY_HASH)
+		{
+			connectionFlags |= CONNECTION_PER_PLACEMENT;
+		}
+
 		connection = GetPlacementConnection(connectionFlags, placement, nodeUser);
 
 		if (PQstatus(connection->pgConn) != CONNECTION_OK)
@@ -895,7 +900,7 @@ OpenCopyConnections(CopyStmt *copyStatement, ShardConnections *shardConnections,
  * worker nodes for user-defined types. If the function can not detect a binary
  * output function for any of the column, it returns false.
  */
-static bool
+bool
 CanUseBinaryCopyFormat(TupleDesc tupleDescription)
 {
 	bool useBinaryCopyFormat = true;
@@ -906,9 +911,6 @@ CanUseBinaryCopyFormat(TupleDesc tupleDescription)
 	{
 		Form_pg_attribute currentColumn = TupleDescAttr(tupleDescription, columnIndex);
 		Oid typeId = InvalidOid;
-		char typeCategory = '\0';
-		bool typePreferred = false;
-		bool binaryOutputFunctionDefined = false;
 
 		if (currentColumn->attisdropped)
 		{
@@ -916,31 +918,48 @@ CanUseBinaryCopyFormat(TupleDesc tupleDescription)
 		}
 
 		typeId = currentColumn->atttypid;
-
-		/* built-in types may also don't have binary output function */
-		binaryOutputFunctionDefined = BinaryOutputFunctionDefined(typeId);
-		if (!binaryOutputFunctionDefined)
+		if (!CanUseBinaryCopyFormatForType(typeId))
 		{
 			useBinaryCopyFormat = false;
 			break;
 		}
-
-		if (typeId >= FirstNormalObjectId)
-		{
-			get_type_category_preferred(typeId, &typeCategory, &typePreferred);
-			if (typeCategory == TYPCATEGORY_ARRAY ||
-				typeCategory == TYPCATEGORY_COMPOSITE)
-			{
-				useBinaryCopyFormat = false;
-				break;
-			}
-		}
 	}
 
 	return useBinaryCopyFormat;
 }
 
 
+/*
+ * CanUseBinaryCopyFormatForType determines whether it is safe to use the
+ * binary copy format for the given type. The binary copy format cannot
+ * be used for arrays or composite types that contain user-defined types,
+ * or when there is no binary output function defined.
+ */
+bool
+CanUseBinaryCopyFormatForType(Oid typeId)
+{
+	if (!BinaryOutputFunctionDefined(typeId))
+	{
+		return false;
+	}
+
+	if (typeId >= FirstNormalObjectId)
+	{
+		char typeCategory = '\0';
+		bool typePreferred = false;
+
+		get_type_category_preferred(typeId, &typeCategory, &typePreferred);
+		if (typeCategory == TYPCATEGORY_ARRAY ||
+			typeCategory == TYPCATEGORY_COMPOSITE)
+		{
+			return false;
+		}
+	}
+
+	return true;
+}
+
+
 /*
  * BinaryOutputFunctionDefined checks whether binary output function is defined
  * for the given type.
@@ -1147,7 +1166,7 @@ SendCopyDataToPlacement(StringInfo dataBuffer, int64 shardId, MultiConnection *c
 	if (!PutRemoteCopyData(connection, dataBuffer->data, dataBuffer->len))
 	{
 		ereport(ERROR, (errcode(ERRCODE_IO_ERROR),
-						errmsg("failed to COPY to shard %ld on %s:%d",
+						errmsg("failed to COPY to shard " INT64_FORMAT " on %s:%d",
 							   shardId, connection->hostname, connection->port),
 						errdetail("failed to send %d bytes %s", dataBuffer->len,
 								  dataBuffer->data)));
@@ -1160,7 +1179,7 @@ SendCopyDataToPlacement(StringInfo dataBuffer, int64 shardId, MultiConnection *c
  * If stopOnFailure is true, then EndRemoteCopy reports an error on failure,
  * otherwise it reports a warning or continues.
  */
-static void
+void
 EndRemoteCopy(int64 shardId, List *connectionList, bool stopOnFailure)
 {
 	ListCell *connectionCell = NULL;
@@ -1174,13 +1193,15 @@ EndRemoteCopy(int64 shardId, List *connectionList, bool stopOnFailure)
 		/* end the COPY input */
 		if (!PutRemoteCopyEnd(connection, NULL))
 		{
-			if (stopOnFailure)
+			if (!stopOnFailure)
 			{
-				ereport(ERROR, (errcode(ERRCODE_IO_ERROR),
-								errmsg("failed to COPY to shard %ld on %s:%d",
-									   shardId, connection->hostname, connection->port)));
+				continue;
 			}
 
+			ereport(ERROR, (errcode(ERRCODE_IO_ERROR),
+							errmsg("failed to COPY to shard " INT64_FORMAT " on %s:%d",
+								   shardId, connection->hostname, connection->port)));
+
 			continue;
 		}
 
@@ -2079,7 +2100,12 @@ CitusCopyDestReceiverStartup(DestReceiver *dest, int operation,
 
 	/* prevent concurrent placement changes and non-commutative DML statements */
 	LockShardListMetadata(shardIntervalList, ShareLock);
-	LockShardListResources(shardIntervalList, ShareLock);
+
+	/*
+	 * Prevent concurrent UPDATE/DELETE on replication factor >1
+	 * (see AcquireExecutorMultiShardLocks() at multi_router_executor.c)
+	 */
+	LockShardListResources(shardIntervalList, RowExclusiveLock);
 
 	/* keep the table metadata to avoid looking it up for every tuple */
 	copyDest->tableMetadata = cacheEntry;
