@@ -599,7 +599,7 @@ LogCancellingBackend(TransactionNode *transactionNode)
 
 	appendStringInfo(logMessage, "Cancelling the following backend "
 								 "to resolve distributed deadlock "
-								 "(transaction numner = %ld, pid = %d)",
+								 "(transaction numner = " UINT64_FORMAT ", pid = %d)",
 					 transactionNode->transactionId.transactionNumber,
 					 transactionNode->initiatorProc->pid);
 
@@ -625,7 +625,8 @@ LogTransactionNode(TransactionNode *transactionNode)
 	logMessage = makeStringInfo();
 	transactionId = &(transactionNode->transactionId);
 
-	appendStringInfo(logMessage, "[DistributedTransactionId: (%d, %ld, %s)] = ",
+	appendStringInfo(logMessage,
+					 "[DistributedTransactionId: (%d, " UINT64_FORMAT ", %s)] = ",
 					 transactionId->initiatorNodeIdentifier,
 					 transactionId->transactionNumber,
 					 timestamptz_to_str(transactionId->timestamp));
@@ -685,7 +686,7 @@ WaitsForToString(List *waitsFor)
 			appendStringInfoString(transactionIdStr, ",");
 		}
 
-		appendStringInfo(transactionIdStr, "%ld",
+		appendStringInfo(transactionIdStr, UINT64_FORMAT,
 						 waitingNode->transactionId.transactionNumber);
 	}
 
