@@ -37,6 +37,7 @@
 #include "distributed/multi_partitioning_utils.h"
 #include "distributed/relay_utility.h"
 #include "distributed/master_metadata_utility.h"
+#include "distributed/version_compat.h"
 #include "foreign/foreign.h"
 #include "lib/stringinfo.h"
 #include "nodes/nodes.h"
@@ -333,7 +334,7 @@ pg_get_tableschemadef_string(Oid tableRelationId, bool includeSequenceDefaults)
 
 	for (attributeIndex = 0; attributeIndex < tupleDescriptor->natts; attributeIndex++)
 	{
-		Form_pg_attribute attributeForm = tupleDescriptor->attrs[attributeIndex];
+		Form_pg_attribute attributeForm = TupleDescAttr(tupleDescriptor, attributeIndex);
 
 		/*
 		 * We disregard the inherited attributes (i.e., attinhcount > 0) here. The
@@ -545,7 +546,7 @@ pg_get_tablecolumnoptionsdef_string(Oid tableRelationId)
 
 	for (attributeIndex = 0; attributeIndex < tupleDescriptor->natts; attributeIndex++)
 	{
-		Form_pg_attribute attributeForm = tupleDescriptor->attrs[attributeIndex];
+		Form_pg_attribute attributeForm = TupleDescAttr(tupleDescriptor, attributeIndex);
 		char *attributeName = NameStr(attributeForm->attname);
 		char defaultStorageType = get_typstorage(attributeForm->atttypid);
 
@@ -1065,3 +1066,49 @@ contain_nextval_expression_walker(Node *node, void *context)
 	}
 	return expression_tree_walker(node, contain_nextval_expression_walker, context);
 }
+
+
+/*
+ * pg_get_replica_identity_command function returns the required ALTER .. TABLE
+ * command to define the replica identity.
+ */
+char *
+pg_get_replica_identity_command(Oid tableRelationId)
+{
+	Relation relation = NULL;
+	StringInfo buf = makeStringInfo();
+	char *relationName = NULL;
+	char replicaIdentity = 0;
+
+	relation = heap_open(tableRelationId, AccessShareLock);
+
+	replicaIdentity = relation->rd_rel->relreplident;
+
+	relationName = generate_qualified_relation_name(tableRelationId);
+
+	if (replicaIdentity == REPLICA_IDENTITY_INDEX)
+	{
+		Oid indexId = RelationGetReplicaIndex(relation);
+
+		if (OidIsValid(indexId))
+		{
+			appendStringInfo(buf, "ALTER TABLE %s REPLICA IDENTITY USING INDEX %s ",
+							 relationName,
+							 quote_identifier(get_rel_name(indexId)));
+		}
+	}
+	else if (replicaIdentity == REPLICA_IDENTITY_NOTHING)
+	{
+		appendStringInfo(buf, "ALTER TABLE %s REPLICA IDENTITY NOTHING",
+						 relationName);
+	}
+	else if (replicaIdentity == REPLICA_IDENTITY_FULL)
+	{
+		appendStringInfo(buf, "ALTER TABLE %s REPLICA IDENTITY FULL",
+						 relationName);
+	}
+
+	heap_close(relation, AccessShareLock);
+
+	return (buf->len > 0) ? buf->data : NULL;
+}
