@@ -7,6 +7,7 @@
  *-------------------------------------------------------------------------
  */
 
+#include "stdint.h"
 #include "postgres.h"
 
 #include "miscadmin.h"
@@ -116,9 +117,13 @@ typedef struct MetadataCacheData
 	Oid distTransactionRelationId;
 	Oid distTransactionGroupIndexId;
 	Oid distTransactionRecordIndexId;
+	Oid copyFormatTypeId;
+	Oid readIntermediateResultFuncId;
 	Oid extraDataContainerFuncId;
 	Oid workerHashFunctionId;
 	Oid extensionOwner;
+	Oid binaryCopyFormatId;
+	Oid textCopyFormatId;
 	Oid primaryNodeRoleId;
 	Oid secondaryNodeRoleId;
 	Oid unavailableNodeRoleId;
@@ -194,6 +199,7 @@ static void CachedRelationLookup(const char *relationName, Oid *cachedOid);
 static ShardPlacement * ResolveGroupShardPlacement(
 	GroupShardPlacement *groupShardPlacement, ShardCacheEntry *shardEntry);
 static WorkerNode * LookupNodeForGroup(uint32 groupid);
+static Oid LookupEnumValueId(Oid typeId, char *valueName);
 
 
 /* exports for SQL callable functions */
@@ -1833,6 +1839,70 @@ DistPlacementGroupidIndexId(void)
 }
 
 
+/* return oid of the read_intermediate_result(text,citus_copy_format) function */
+Oid
+CitusReadIntermediateResultFuncId(void)
+{
+	if (MetadataCache.readIntermediateResultFuncId == InvalidOid)
+	{
+		List *functionNameList = list_make2(makeString("pg_catalog"),
+											makeString("read_intermediate_result"));
+		Oid copyFormatTypeOid = CitusCopyFormatTypeId();
+		Oid paramOids[2] = { TEXTOID, copyFormatTypeOid };
+		bool missingOK = false;
+
+		MetadataCache.readIntermediateResultFuncId =
+			LookupFuncName(functionNameList, 2, paramOids, missingOK);
+	}
+
+	return MetadataCache.readIntermediateResultFuncId;
+}
+
+
+/* return oid of the citus.copy_format enum type */
+Oid
+CitusCopyFormatTypeId(void)
+{
+	if (MetadataCache.copyFormatTypeId == InvalidOid)
+	{
+		char *typeName = "citus_copy_format";
+		MetadataCache.copyFormatTypeId = GetSysCacheOid2(TYPENAMENSP,
+														 PointerGetDatum(typeName),
+														 PG_CATALOG_NAMESPACE);
+	}
+
+	return MetadataCache.copyFormatTypeId;
+}
+
+
+/* return oid of the 'binary' citus_copy_format enum value */
+Oid
+BinaryCopyFormatId(void)
+{
+	if (MetadataCache.binaryCopyFormatId == InvalidOid)
+	{
+		Oid copyFormatTypeId = CitusCopyFormatTypeId();
+		MetadataCache.binaryCopyFormatId = LookupEnumValueId(copyFormatTypeId, "binary");
+	}
+
+	return MetadataCache.binaryCopyFormatId;
+}
+
+
+/* return oid of the 'text' citus_copy_format enum value */
+Oid
+TextCopyFormatId(void)
+{
+	if (MetadataCache.textCopyFormatId == InvalidOid)
+	{
+		Oid copyFormatTypeId = CitusCopyFormatTypeId();
+		MetadataCache.textCopyFormatId = LookupEnumValueId(copyFormatTypeId, "text");
+	}
+
+	return MetadataCache.textCopyFormatId;
+}
+
+
 /* return oid of the citus_extradata_container(internal) function */
 Oid
 CitusExtraDataContainerFuncId(void)
@@ -2001,17 +2071,27 @@ LookupNodeRoleValueId(char *valueName)
 	}
 	else
 	{
-		Datum nodeRoleIdDatum = ObjectIdGetDatum(nodeRoleTypId);
-		Datum valueDatum = CStringGetDatum(valueName);
-
-		Datum valueIdDatum = DirectFunctionCall2(enum_in, valueDatum, nodeRoleIdDatum);
-
-		Oid valueId = DatumGetObjectId(valueIdDatum);
+		Oid valueId = LookupEnumValueId(nodeRoleTypId, valueName);
 		return valueId;
 	}
 }
 
 
+/*
+ * LookupEnumValueId looks up the OID of an enum value.
+ */
+static Oid
+LookupEnumValueId(Oid typeId, char *valueName)
+{
+	Datum typeIdDatum = ObjectIdGetDatum(typeId);
+	Datum valueDatum = CStringGetDatum(valueName);
+	Datum valueIdDatum = DirectFunctionCall2(enum_in, valueDatum, typeIdDatum);
+	Oid valueId = DatumGetObjectId(valueIdDatum);
+
+	return valueId;
+}
+
+
 /* return the Oid of the 'primary' nodeRole enum value */
 Oid
 PrimaryNodeRoleId(void)
