@@ -55,7 +55,8 @@ typedef enum CitusNodeTag
 	T_MultiExtendedOp,
 	T_Job,
 	T_MapMergeJob,
-	T_MultiPlan,
+	T_DistributedPlan,
+	T_DistributedSubPlan,
 	T_Task,
 	T_TaskExecution,
 	T_ShardInterval,
