@@ -2,7 +2,7 @@
 -- MULTI_FOREIGN_KEY
 --
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1350000;
+SET citus.next_shard_id TO 1350000;
 
 -- set shard_count to 4 for faster tests, because we create/drop lots of shards in this test.
 SET citus.shard_count TO 4;
