@@ -6,7 +6,7 @@
 -- need to increase the logging verbosity of messages displayed on the client.
 
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 770000;
+SET citus.next_shard_id TO 770000;
 
 -- Adding additional l_orderkey = 1 to make this query not router executable
 SELECT l_orderkey, l_linenumber, l_shipdate FROM lineitem WHERE l_orderkey = 9030 or l_orderkey = 1;
