@@ -55,6 +55,92 @@ WHERE
       )
 LIMIT 3;
 
+-- immutable functions are also treated as reference tables
+SELECT
+  user_id
+FROM
+  (SELECT user_id FROM generate_series(1,10) AS series(user_id)) users_reference_table
+WHERE
+  NOT EXISTS
+      (SELECT
+          value_2
+       FROM
+          events_table
+       WHERE
+          users_reference_table.user_id = events_table.user_id
+      )
+LIMIT 3;
+
+-- subqueries without FROM are also treated as reference tables
+SELECT
+  user_id
+FROM
+  (SELECT  5 AS user_id) users_reference_table
+WHERE
+  NOT EXISTS
+      (SELECT
+          value_2
+       FROM
+          events_table
+       WHERE
+          users_reference_table.user_id = events_table.user_id
+      )
+LIMIT 3;
+
+-- join with distributed table prevents FROM from recurring
+SELECT
+  DISTINCT user_id
+FROM
+  (SELECT s FROM generate_series(1,10) s) series,
+  (SELECT DISTINCT user_id FROM users_table) users_table,
+  (SELECT 1 AS one) one
+WHERE
+  s = user_id AND user_id > one AND
+  user_id IN
+      (SELECT
+          value_2
+       FROM
+          events_table
+       WHERE
+          users_table.user_id = events_table.user_id
+      )
+ORDER BY user_id
+LIMIT 3;
+
+-- inner join between distributed prevents FROM from recurring
+SELECT
+  DISTINCT user_id
+FROM
+  users_table JOIN users_reference_table USING (user_id)
+WHERE
+  users_table.value_2 IN
+      (SELECT
+          value_2
+       FROM
+          events_table
+       WHERE
+          users_table.user_id = events_table.user_id
+      )
+ORDER BY user_id
+LIMIT 3;
+
+-- outer join could still recur
+SELECT
+  DISTINCT user_id
+FROM
+  users_table RIGHT JOIN users_reference_table USING (user_id)
+WHERE
+  users_table.value_2 IN
+      (SELECT
+          value_2
+       FROM
+          events_table
+       WHERE
+          users_table.user_id = events_table.user_id
+      )
+ORDER BY user_id
+LIMIT 3;
+
 -- subqueries in WHERE with IN operator without equality
 SELECT 
   users_table.user_id, count(*)
@@ -73,10 +159,47 @@ GROUP BY users_table.user_id
 ORDER BY 2 DESC, 1 DESC
 LIMIT 3;
 
+-- immutable functions are also treated as reference tables
+SELECT
+  users_table.user_id, count(*)
+FROM
+  users_table
+WHERE
+  value_2 IN
+          (SELECT
+              value_2
+           FROM
+              generate_series(1,10) AS events_reference_table(user_id)
+           WHERE
+              users_table.user_id > events_reference_table.user_id
+          )
+GROUP BY users_table.user_id
+ORDER BY 2 DESC, 1 DESC
+LIMIT 3;
+
+-- immutable functions are also treated as reference tables
+SELECT
+  users_table.user_id, count(*)
+FROM
+  users_table
+WHERE
+  value_2 IN
+          (SELECT
+              value_2
+           FROM
+              (SELECT 5 AS user_id) AS events_reference_table
+           WHERE
+              users_table.user_id > events_reference_table.user_id
+          )
+GROUP BY users_table.user_id
+ORDER BY 2 DESC, 1 DESC
+LIMIT 3;
+
+
 -- should error out since reference table exist on the left side 
 -- of the left lateral join
 SELECT user_id, value_2 FROM users_table WHERE
-  value_1 > 101 AND value_1 < 110
+  value_1 > 1 AND value_1 < 3
   AND value_2 >= 5
   AND user_id IN
   (
@@ -90,7 +213,7 @@ SELECT user_id, value_2 FROM users_table WHERE
 		    min(time) AS view_homepage_time
 		  FROM events_reference_table
 		     WHERE
-		     event_type IN (10, 20, 30, 40, 50, 60, 70, 80, 90)
+		     event_type IN (1, 2)
 		  GROUP BY user_id
 		) e1 LEFT JOIN LATERAL (
 		  SELECT
@@ -100,7 +223,7 @@ SELECT user_id, value_2 FROM users_table WHERE
 		  FROM events_reference_table
 		  WHERE
 		    user_id = e1.user_id AND
-		       event_type IN (11, 21, 31, 41, 51, 61, 71, 81, 91)
+		       event_type IN (2, 3)
 		  ORDER BY time
 		) e2 ON true LEFT JOIN LATERAL (
 		  SELECT
@@ -110,7 +233,7 @@ SELECT user_id, value_2 FROM users_table WHERE
 		  FROM  events_reference_table
 		  WHERE
 		    user_id = e2.user_id AND
-		    event_type IN (12, 22, 32, 42, 52, 62, 72, 82, 92)
+		    event_type IN (3, 4)
 		  ORDER BY time
 		) e3 ON true LEFT JOIN LATERAL (
 		  SELECT
@@ -120,7 +243,7 @@ SELECT user_id, value_2 FROM users_table WHERE
 		  FROM  events_reference_table
 		  WHERE
 		    user_id = e3.user_id AND
-		    event_type IN (13, 23, 33, 43, 53, 63, 73, 83, 93)
+		    event_type IN (4, 5)
 		  ORDER BY time
 		) e4 ON true LEFT JOIN LATERAL (
 		  SELECT
@@ -128,7 +251,7 @@ SELECT user_id, value_2 FROM users_table WHERE
 		  FROM  events_reference_table
 		  WHERE
 		    user_id = e4.user_id AND
-		    event_type IN (14, 24, 34, 44, 54, 64, 74, 84, 94)
+		    event_type IN (5, 6)
 		  ORDER BY time
 		) e5 ON true
 		group by e1.user_id
@@ -142,7 +265,7 @@ ORDER BY 1, 2;
 FROM 
   users_table 
 WHERE 
-  value_3 =ANY(SELECT value_2 FROM users_reference_table WHERE value_1 >= 10 AND value_1 <= 20) 
+  value_3 =ANY(SELECT value_2 FROM users_reference_table WHERE value_1 >= 1 AND value_1 <= 2) 
  GROUP BY 1 ORDER BY 2 DESC, 1 DESC LIMIT 5;
 
 
@@ -158,8 +281,8 @@ WHERE
              FROM 
               events_reference_table as e2
              WHERE
-              value_2 = 15 AND
-              value_3 > 25 AND
+              value_2 = 2 AND
+              value_3 > 3 AND
               e1.value_2 > e2.value_2
             ) 
 GROUP BY 1
@@ -170,7 +293,7 @@ LIMIT 5;
 -- should work since reference table is on the 
 -- inner part of the join 
 SELECT user_id, value_2 FROM users_table WHERE
-  value_1 > 101 AND value_1 < 110
+  value_1 > 1 AND value_1 < 3
   AND value_2 >= 5
   AND user_id IN
   (
@@ -184,7 +307,7 @@ SELECT user_id, value_2 FROM users_table WHERE
         min(time) AS view_homepage_time
       FROM events_table
          WHERE
-         event_type IN (10, 20, 30, 40, 50, 60, 70, 80, 90)
+         event_type IN (1, 2)
       GROUP BY user_id
     ) e1 LEFT JOIN LATERAL (
       SELECT
@@ -194,7 +317,7 @@ SELECT user_id, value_2 FROM users_table WHERE
       FROM events_table
       WHERE
         user_id = e1.user_id AND
-           event_type IN (11, 21, 31, 41, 51, 61, 71, 81, 91)
+           event_type IN (2, 3)
       ORDER BY time
     ) e2 ON true LEFT JOIN LATERAL (
       SELECT
@@ -204,7 +327,7 @@ SELECT user_id, value_2 FROM users_table WHERE
       FROM  events_table
       WHERE
         user_id = e2.user_id AND
-        event_type IN (12, 22, 32, 42, 52, 62, 72, 82, 92)
+        event_type IN (3, 4)
       ORDER BY time
     ) e3 ON true LEFT JOIN LATERAL (
       SELECT
@@ -214,7 +337,7 @@ SELECT user_id, value_2 FROM users_table WHERE
       FROM  events_table
       WHERE
         user_id = e3.user_id AND
-        event_type IN (13, 23, 33, 43, 53, 63, 73, 83, 93)
+        event_type IN (4, 5)
       ORDER BY time
     ) e4 ON true LEFT JOIN LATERAL (
       SELECT
@@ -222,7 +345,7 @@ SELECT user_id, value_2 FROM users_table WHERE
       FROM  events_reference_table
       WHERE
         user_id = e4.user_id AND
-        event_type IN (14, 24, 34, 44, 54, 64, 74, 84, 94)
+        event_type IN (5, 6)
       ORDER BY time
     ) e5 ON true
     group by e1.user_id
@@ -257,10 +380,10 @@ FROM users_reference_table
 WHERE value_2 > ALL
     (SELECT min(value_2)
      FROM events_table
-     WHERE event_type > 50 AND users_reference_table.user_id = events_table.user_id
+     WHERE event_type > 2 AND users_reference_table.user_id = events_table.user_id
      GROUP BY user_id)
 GROUP BY user_id
-HAVING count(*) > 66
+HAVING count(*) > 3
 ORDER BY 2 DESC,
          1 DESC
 LIMIT 5;
@@ -275,7 +398,9 @@ WHERE user_id IN
           users_reference_table
      WHERE users_reference_table.user_id NOT IN
          (SELECT value_2
-          FROM users_reference_table AS u2));
+          FROM users_reference_table AS u2))
+ORDER BY 1,2,3
+LIMIT 5;
 
 -- not supported since GROUP BY references to an upper level query
 SELECT 
@@ -289,11 +414,33 @@ WHERE
            FROM 
               events_reference_table  
            WHERE 
-              users_table.user_id = events_reference_table.user_id AND event_type = 50
+              users_table.user_id = events_reference_table.user_id AND event_type = 2
            GROUP BY
               users_table.user_id
           )
 GROUP BY user_id
-HAVING count(*) > 66
+HAVING count(*) > 3
+ORDER BY user_id
+LIMIT 5;
+
+-- similar query with slightly more complex group by
+-- though the error message is a bit confusing
+SELECT 
+  user_id
+FROM 
+  users_table
+WHERE 
+  value_2 >  
+          (SELECT 
+              max(value_2) 
+           FROM 
+              events_reference_table  
+           WHERE 
+              users_table.user_id = events_reference_table.user_id AND event_type = 2
+           GROUP BY
+              (users_table.user_id * 2)
+          )
+GROUP BY user_id
+HAVING count(*) > 3
 ORDER BY user_id
 LIMIT 5;
