@@ -1,5 +1,5 @@
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 990000;
+SET citus.next_shard_id TO 990000;
 
 
 -- ===================================================================
@@ -20,6 +20,12 @@ COPY sharded_table TO STDOUT;
 COPY (SELECT COUNT(*) FROM sharded_table) TO STDOUT;
 COMMIT;
 
+-- ANALYZE is supported in a transaction block
+BEGIN;
+ANALYZE sharded_table;
+ANALYZE sharded_table;
+END;
+
 -- cursors may not involve distributed tables
 DECLARE all_sharded_rows CURSOR FOR SELECT * FROM sharded_table;
 
@@ -145,6 +151,46 @@ begin
 end
 $$ language plpgsql;
 
+\c - - - :worker_2_port
+CREATE MATERIALIZED VIEW prevcounts AS
+SELECT analyze_count, vacuum_count FROM pg_stat_user_tables
+WHERE relname='dustbunnies_990001';
+-- create function that sleeps until those counters increment
+create function wait_for_stats() returns void as $$
+declare
+  start_time timestamptz := clock_timestamp();
+  analyze_updated bool;
+  vacuum_updated bool;
+begin
+  -- we don't want to wait forever; loop will exit after 10 seconds
+  for i in 1 .. 100 loop
+
+    -- check to see if analyze has been updated
+    SELECT (st.analyze_count >= pc.analyze_count + 1) INTO analyze_updated
+      FROM pg_stat_user_tables AS st, pg_class AS cl, prevcounts AS pc
+     WHERE st.relname='dustbunnies_990001' AND cl.relname='dustbunnies_990001';
+
+     -- check to see if vacuum has been updated
+    SELECT (st.vacuum_count >= pc.vacuum_count + 1) INTO vacuum_updated
+      FROM pg_stat_user_tables AS st, pg_class AS cl, prevcounts AS pc
+     WHERE st.relname='dustbunnies_990001' AND cl.relname='dustbunnies_990001';
+
+    exit when analyze_updated or vacuum_updated;
+
+    -- wait a little
+    perform pg_sleep(0.1);
+
+    -- reset stats snapshot so we can test again
+    perform pg_stat_clear_snapshot();
+
+  end loop;
+
+  -- report time waited in postmaster log (where it won't change test output)
+  raise log 'wait_for_stats delayed % seconds',
+    extract(epoch from clock_timestamp() - start_time);
+end
+$$ language plpgsql;
+
 -- run VACUUM and ANALYZE against the table on the master
 \c - - - :master_port
 VACUUM dustbunnies;
@@ -209,11 +255,22 @@ WHERE tablename = 'dustbunnies_990002' ORDER BY attname;
 -- verify warning for unqualified VACUUM
 VACUUM;
 
+-- check the current number of vacuum and analyze run on dustbunnies
+SELECT run_command_on_workers($$SELECT wait_for_stats()$$);
+SELECT run_command_on_workers($$SELECT pg_stat_get_vacuum_count(tablename::regclass) from pg_tables where tablename LIKE 'dustbunnies_%' limit 1$$);
+SELECT run_command_on_workers($$SELECT pg_stat_get_analyze_count(tablename::regclass) from pg_tables where tablename LIKE 'dustbunnies_%' limit 1$$);
+
 -- and warning when using targeted VACUUM without DDL propagation
 SET citus.enable_ddl_propagation to false;
 VACUUM dustbunnies;
+ANALYZE dustbunnies;
 SET citus.enable_ddl_propagation to DEFAULT;
 
+-- should not propagate the vacuum and analyze
+SELECT run_command_on_workers($$SELECT wait_for_stats()$$);
+SELECT run_command_on_workers($$SELECT pg_stat_get_vacuum_count(tablename::regclass) from pg_tables where tablename LIKE 'dustbunnies_%' limit 1$$);
+SELECT run_command_on_workers($$SELECT pg_stat_get_analyze_count(tablename::regclass) from pg_tables where tablename LIKE 'dustbunnies_%' limit 1$$);
+
 -- test worker_hash
 SELECT worker_hash(123);
 SELECT worker_hash('1997-08-08'::date);
